
from _wrapper import Wrapper
from _plot import plot, html



class Highcharts(Wrapper):
    """
    Main Highcharts Object

    API: http://api.highcharts.com/highcharts
    Demos: http://www.highcharts.com/demo
    """

    def __init__(self):
        Wrapper.__init__(self, lib='highcharts')

        #default config
        self.credits.enabled = False
        self.exporting.enabled = True
        self.chart.animation = False
        self.series.animation = False

        self.plotOptions.series.animation = False
        self.plotOptions.line.animation = False
        self.plotOptions.column.animation = False
        self.plotOptions.bar.animation = False
        self.plotOptions.pie.animation = False
        self.plotOptions.scatter.animation = False
        self.plotOptions.bubble.animation = False
        self.plotOptions.treemap.animation = False


    def plot(self, dated=True, save=False, save_name=None, html_init=None,
             js_option_postprocess=None, js_extra=None, callback=None):
        opt = self.to_dict()
        return plot(opt, lib='highcharts', dated=dated, save=save, save_name=save_name,
            html_init=html_init, js_option_postprocess=js_option_postprocess,
            js_extra=js_extra, callback=callback)

    def html(self, dated=True, save=False, save_name=None, html_init=None,
             js_option_postprocess=None, js_extra=None, callback=None):
        opt = self.to_dict()
        return html(opt, lib='highcharts', dated=dated, save=save, save_name=save_name,
            html_init=html_init, js_option_postprocess=js_option_postprocess,
            js_extra=js_extra, callback=callback)




import os
import pandas as pd
import datetime as dt
import uuid
from IPython.display import HTML


from _config import JS_LIBS_ONE, JS_LIBS_TWO, JS_SAVE
from scripts import JS_JSON_PARSE



def html(options, lib='hicharts', dated=True, save=False, save_name=None,
         html_init=None, js_option_postprocess=None, js_extra=None, callback=None, footer=None):
    """
    save=True will create a standalone HTML doc under localdir/saved (creating folfer save if necessary)
    """

    def json_dumps(obj):
        return pd.io.json.dumps(obj)


    chart_id = str(uuid.uuid4()).replace('-', '_')

    _options = dict(options)
    _options['chart']['renderTo'] = chart_id+'container_chart'
    json_options = json_dumps(_options).replace('__uuid__', chart_id)


    # HTML
    html_init = html_init if html_init else '<div id="__uuid__"><div id="__uuid__container_chart"></div></div>'
    footer = footer if footer else ''
    
    html = html_init + footer
    html = html.replace('__uuid__', chart_id)


    # JS
    js_option_postprocess = js_option_postprocess.replace('__uuid__', chart_id) if js_option_postprocess else ''
    js_extra = js_extra.replace('__uuid__', chart_id) if js_extra else ''
    callback = ', '+callback.replace('__uuid__', chart_id) if callback else ''

    js_option = """
    var options = %s;
    %s
    %s
    window.opt = $.extend(true, {}, options);
    console.log('Highcharts/Highstock options accessible as opt');
    """ % (json_options, JS_JSON_PARSE, js_option_postprocess)


    if lib=='highcharts':
        js_call = 'window.chart = new Highcharts.Chart(options%s);' % (callback)
    elif lib=='highstock':
        js_call = 'window.chart = new Highcharts.StockChart(options%s);' % (callback)

    js_debug = """
    console.log('Highcharts/Highstock chart accessible as chart');
    """

    js = """<script>
    // nbconvert loads jquery.min.js  and require.js and at the top of the .ipnb
    // then to make jquery available inside a require module
    // the trick is http://www.manuel-strehl.de/dev/load_jquery_before_requirejs.en.html
    define('jquery', [], function() {
    return jQuery;
    });

    require(%s, function() {
        require(%s, function() {
            %s
            %s
            %s
            %s
        });
    });
    </script>""" % (JS_LIBS_ONE, JS_LIBS_TWO, js_option, js_extra, js_call, js_debug)


    # save
    if save==True:
        if not os.path.exists('saved'):
            os.makedirs('saved')
        tag = save_name if save_name else 'plot'
        dated = dt.datetime.now().strftime('_%Y%m%d_%H%M%S') if dated else ''
        with open(os.path.join('saved', tag+dated+'.html'), 'w') as f:
            contents = """
            <script src="%s"></script>
            <script src="%s"></script>
            %s
            """ % (JS_SAVE[0], JS_SAVE[1], html+js)
            f.write(contents)

    return html+js





def plot(options, lib='hicharts', dated=True, save=False, save_name=None,
         html_init=None, js_option_postprocess=None, js_extra=None, callback=None, footer=None):
    contents = html(options, lib, dated, save, save_name, html_init, js_option_postprocess, js_extra, callback, footer)
    return HTML(contents)








from _config import load_js_libs


from ._highcharts import Highcharts
from ._highstock import Highstock
import sample
import build
from ._clock import Clock


__all__ = ['Highcharts',
           'Highstock',
           'samples',
           'build',
           'Clock',
           ]


load_js_libs()
print 'Open console to check js libs were loaded in page'

__author__ = 'oscar6eho'



# necessary to push to PyPI
# cf. http://peterdowns.com/posts/first-time-with-pypi.html
# cf. https://tom-christie.github.io/articles/pypi/
# cf. https://pythonhosted.org/setuptools/setuptools.html


from setuptools import setup

with open('README.rst') as f:
    long_description = f.read()


setup(
  name = 'ezhc',
  packages = ['ezhc'],
  version = '0.5.11',
  description = 'easy Highcharts & Highstock, dynamic plots from pandas dataframes in the Jupyter notebook',
  long_description = long_description,
  author = 'oscar6echo',
  author_email = 'olivier.borderies@gmail.com',
  url = 'https://github.com/oscar6echo/ezhc', # use the URL to the github repo
  download_url = 'https://github.com/oscar6echo/ezhc/tarball/0.5.11', # tag number at the end
  keywords = ['Highcharts', 'Highstock', 'pandas', 'notebook', 'javascript'], # arbitrary keywords
  license='MIT',
  classifiers = [ 'Development Status :: 4 - Beta',
                  'License :: OSI Approved :: MIT License',
                  'Programming Language :: Python :: 2.7'
  ],
  install_requires = [
    'pandas>=0.17',
    'jinja2>=2.8',
    'ipython>=3.0',
    'requests>=2.9'
  ],
  include_package_data=True,
  package_data={
    'api':
         ['api/hc_object.json',
          'api/hc_option.json',
          'api/hs_object.json',
          'api/hs_option.json'
         ],
    'script':
        ['script/financial_time_series_table_options.js',
         'script/financial_time_series_0.js',
         'script/financial_time_series_table_1.js',
         'script/financial_time_series_table_2.js',
         'script/formatter_basic.js',
         'script/formatter_percent.js',
         'script/formatter_quantile.js',
         'script/json_parse.js',
         'script/tooltip_positioner_center_top.js',
         'script/tooltip_positioner_left_top.js',
         'script/tooltip_positioner_right_top.js',
         'script/template_disclaimer.html',
         'script/Jupyter_logo.png',
         'script/SG_logo.png'
        ],
    'samples':
        ['samples/df_bubble.csv',
         'samples/df_heatmap.csv',
         'samples/df_one_idx_one_col.csv',
         'samples/df_one_idx_several_col_2.csv',
         'samples/df_one_idx_several_col.csv',
         'samples/df_one_idx_two_col.csv',
         'samples/df_scatter.csv',
         'samples/df_several_idx_one_col.csv',
         'samples/df_two_idx_one_col.csv'
        ]
    },
)





import requests
import base64

requests.packages.urllib3.disable_warnings()


def is_http_url(img_path):
    img_path = img_path.lower()
    return (img_path.startswith("http://") or
            img_path.startswith("https://"))

def image_base64(imgpath):
    with open(imgpath, 'rb') as obj:
        return base64.b64encode(obj.read())


def image_content(img_path):
    """Get the image content according to its path
    Different case:
      - PNG,JPEG: convert it to base64
      - SVG: open it and read it
      - HTTP URL: get the content via requests.get
    """
    if is_http_url(img_path):
        content = requests.get(img_path, verify=False).content
        if img_path.endswith(".svg"):
            return content
        return base64.b64encode(content)
    if img_path.endswith(".svg"):
        return open(img_path).read()
    return image_base64(img_path)


def image_src(img_path):
    content = image_content(img_path)
    if img_path.endswith('.svg'):
        return '{}'.format(content)
    else:
        return 'data:image/png;base64,{}'.format(content)





import os
import json
from _config import HC_OPTION_FILE, HC_OBJECT_FILE, \
                    HS_OPTION_FILE, HS_OBJECT_FILE, \
                    API_DIR


def load_resource(src):
    _dir = os.path.dirname(__file__)
    object_builder_file = os.path.join(_dir, API_DIR, src)
    with open(object_builder_file) as obj_build:
        return json.load(obj_build)


class State(object):
    """
    Contains all 'highcharts' or 'highstock' API
    Select lib='highcharts' or lib='highstock'
    """

    def __init__(self, lib='highcharts'):
        if lib=='highcharts':
            self._OBJECT = load_resource(HC_OBJECT_FILE)
            self._OPTION = load_resource(HC_OPTION_FILE)
        elif lib=='highstock':        
            self._OBJECT = load_resource(HS_OBJECT_FILE)
            self._OPTION = load_resource(HS_OPTION_FILE)
        else:
            raise ValueError("Wrong lib, 'highcharts' or 'highstock' expected")

state_HC = State(lib='highcharts')
state_HS= State(lib='highstock')

